﻿function TDIsEnchanter()
	local prof1, prof2 = GetProfessions();

	if prof1 then
		local _, _, _, _, _, _, skillLine1 = GetProfessionInfo(prof1);
		if skillLine1 == 333 then
			return true;
		end
	end

	if prof2 then
		local _, _, _, _, _, _, skillLine2 = GetProfessionInfo(prof2);
		if skillLine2 == 333 then
			return true;
		end
	end

	return false;
end

function TDAutoVendor(maxValue, priceSource, maxQuality, maxIlvl, sellCrapBlue, debug, disenchant, valueDiff)
	maxValue = maxValue or 5000000;
	priceSource = priceSource or 'DBGlobalSaleAvg';
	maxQuality = maxQuality or 3;
	sellCrapBlue = sellCrapBlue or true;
	debug = debug or false;
	disenchant = disenchant or false;
	valueDiff = valueDiff or 100000;
	maxIlvl = maxIlvl or 630;
	-- 0 = junk, 1 = normal, 2 = green, 3 = blue, 4 = purple, 5 = legendary

	local hasEnchanting = TDIsEnchanter();

	if debug then
		print('max value: ' .. maxValue .. ', source: ' .. priceSource .. ', maxQ: ' .. maxQuality .. ', sell blue: '
				.. tostring(sellCrapBlue) .. ', maxIlvl: ' .. maxIlvl .. ', has ench: ' .. tostring(hasEnchanting));
	end

	local WEAPON, ARMOR = TSMAPI.Util:Select({ 1, 2 }, GetAuctionItemClasses());

	for bag, slot, itemString in TSMAPI.Inventory:BagIterator(nil, true) do
		local _, itemLink, quality, ilvl, _, iType, _, _, _, _, vendorPrice = TSMAPI.Item:GetInfo(itemString);
		local disenchantable = TSMAPI.Item:IsDisenchantable(itemString);
		local soulbound = TSMAPI.Item:IsSoulbound(itemString);
		local customSalePrice = TSMAPI:GetCustomPriceValue(priceSource, itemString);
		local disenchantPrice = TSMAPI:GetCustomPriceValue('Destroy', itemString);

		if itemString and vendorPrice > 0 and
			(quality == 0 or -- junk
			(disenchantable and ilvl <= maxIlvl and (iType == ARMOR or iType == WEAPON) and quality <= maxQuality) and
			(not disenchant or not (vendorPrice ~= nil and disenchantPrice ~= nil and
				(disenchantPrice - vendorPrice >= valueDiff))) and
			((customSalePrice and customSalePrice <= maxValue) or
				(not hasEnchanting and sellCrapBlue and soulbound and quality == 3 and
					ilvl >= 570 and ilvl <= 630 and (not customSalePrice or customSalePrice <= maxValue))))
		then
			local venPrice = '';
			local cusPrice = '';
			if vendorPrice > 0 then
				venPrice = ' V: ' .. TSMAPI:MoneyToString(vendorPrice, 'OPT_ICON');
			end
			if disenchant and disenchantPrice and disenchantPrice > 0 then
				cusPrice = ' D: ' .. TSMAPI:MoneyToString(disenchantPrice, 'OPT_ICON');
			end
			if customSalePrice and customSalePrice > 0 then
				cusPrice = cusPrice .. ' AH: ' .. TSMAPI:MoneyToString(customSalePrice, 'OPT_ICON');
			end


			if debug then
				print('Would sell ' .. itemLink .. venPrice .. cusPrice);
			elseif MerchantFrame:IsShown() then
				print('Selling ' .. itemLink .. venPrice .. cusPrice);
				ShowContainerSellCursor(bag, slot);
				UseContainerItem(bag, slot);
				ResetCursor();
			else
				print('Merchant frame is not opened!');
			end
		end
	end
end

function TDAutoDisenchant(debug, maxValue, valueDiff, priceSource, maxQuality, maxIlvl)
	maxValue = maxValue or 5000000;
	valueDiff = valueDiff or 100000;
	priceSource = priceSource or 'DBGlobalSaleAvg';
	maxQuality = maxQuality or 3;
	debug = debug or false;
	maxIlvl = maxIlvl or 630;
	-- 0 = junk, 1 = normal, 2 = green, 3 = blue, 4 = purple, 5 = legendary

	if not TDIsEnchanter() then
		return;
	end

	if debug then
		print('value diff: ' .. valueDiff .. ', source: ' .. priceSource .. ', maxQ: ' .. maxQuality ..
				', maxIlvl: ' .. maxIlvl);
	end

	for bag, slot, itemString in TSMAPI.Inventory:BagIterator(nil, true) do
		local _, itemLink, quality, ilvl, _, _, _, _, _, _, vendorPrice = TSMAPI.Item:GetInfo(itemString);
		local disenchantable = TSMAPI.Item:IsDisenchantable(itemString);
		local customSalePrice = TSMAPI:GetCustomPriceValue(priceSource, itemString);
		local disenchantPrice = TSMAPI:GetCustomPriceValue('Destroy', itemString);

		if itemString and disenchantable and ilvl <= maxIlvl and quality <= maxQuality and
			(not customSalePrice or customSalePrice <= maxValue) and
			(vendorPrice ~= nil and disenchantPrice ~= nil and (disenchantPrice - vendorPrice >= valueDiff))
		then
			local venPrice = '';
			local cusPrice = '';
			if vendorPrice > 0 then
				venPrice = ' V: ' .. TSMAPI:MoneyToString(vendorPrice, 'OPT_ICON');
			end
			if disenchantPrice and disenchantPrice > 0 then
				cusPrice = ' D: ' .. TSMAPI:MoneyToString(disenchantPrice, 'OPT_ICON');
			end

			if debug then
				print('Would dis ' .. itemLink .. venPrice .. cusPrice);
			else
				print('Dissing ' .. itemLink .. venPrice .. cusPrice);
				TDDisButton:SetAttribute('macrotext', format('/cast Disenchant\n/use %d %d', bag, slot));
				return;
			end
		end
	end
end

local autoExcluded = {
	79249 -- tome of clear mind
};

function TDAutoAH(debug, AHToon, priceSource, exclude)
	priceSource = priceSource or 'DBGlobalSaleAvg';
	exclude = exclude or {};
	debug = debug or false;
	if type(exclude) ~= 'table' then
		exclude = {};
	end

	if not AHToon then
		print('Invalid macro, AHToon parameter is required and needs to be in string ex. \'Banker\'');
		return;
	end

	if not MailFrame:IsShown() then
		print('You need to open mail frame first');
		return;
	end

	local mailItemCount = 0;
	local itemLinks = '';
	local CONSUMABLE = TSMAPI.Util:Select({ 4 }, GetAuctionItemClasses());

	for bag, slot, itemString, quantity in TSMAPI.Inventory:BagIterator(nil, false) do
		local customSalePrice = TSMAPI:GetCustomPriceValue(priceSource, itemString);
		local _, itemLink, quality, ilvl, _, iType = TSMAPI.Item:GetInfo(itemString);
		local itemId = TSMAPI.Item:ToItemID(itemString);

		if customSalePrice and iType ~= CONSUMABLE and not tContains(exclude, itemId) and
				not tContains(autoExcluded, itemId) then
			itemLinks = itemLinks .. ' ' .. itemLink;
			PickupContainerItem(bag, slot);
			ClickSendMailItemButton();
			mailItemCount = mailItemCount + 1;
			if mailItemCount >= ATTACHMENTS_MAX_SEND then
				if debug then
					print('Would send: ' .. itemLinks);
					return;
				else
					SendMailNameEditBox:SetText(AHToon);
					SendMail(AHToon, "AH Items", "");
					mailItemCount = 0;
					print('Items sent: ' .. itemLinks);
					itemLinks = '';
				end
			end
		end
	end

	if mailItemCount > 0 then
		if debug then
			print('Would send: ' .. itemLinks);
			return;
		else
			SendMailNameEditBox:SetText(AHToon);
			SendMail(AHToon, "AH Items", "");
			mailItemCount = 0;
			print('Items sent: ' .. itemLinks);
			itemLinks = '';
		end
	end
end

TDOpen_Items = {
	114100, 114105, 114097, 114099, 114094, 114108, 114096, 114098, 114101,
	-- Tormented
	114111, 123975, 114088, 114090, 114091, 114092, 114093, 114089, 119115, 119117, 119119, 119121, 119123, 119125,
	-- Formidable
	124551, 124552, 124553, 124557, 124554, 124555, 124556, 124558, 124559, 124560, 124561, 124562, 124550, 128348,
	-- Baleful
	119114, 123975, 119116, 119118, 119120, 119122, 119124, 114082, 114084, 114085, 114086, 114087, 114112, 114083,
	-- Grandoise
	114110, 114069, 114078, 114079, 118924, 114072, 114076, 114073, 114074, 114077, 114070, 114075, 114071, 114080,
	-- Turbulent
	114109, 114057, 114066, 114060, 114058, 114063, 114059, 114068,
	-- Munificent
	120302, 120301, 123858,
	-- Enhancement Token, Follower Retraining Scroll Case
	123857, 118529, 118531, 118530, 127853, 127854, 127855, 128391, 122484, 122486, 122485,
	-- Runic Pouch, Cache of Highmaul Treasures, Iron Fleet Treasure Chest, Blackrock Foundry Spoils
};

function TDAutoOpen(debug, customList)
	debug = debug or false;
	customList = customList or {};
	if type(customList) ~= 'table' then
		customList = {};
	end
	-- 0 = junk, 1 = normal, 2 = green, 3 = blue, 4 = purple, 5 = legendary

	for bag, slot, itemString in TSMAPI.Inventory:BagIterator(nil, true, true) do
		local name, itemLink, quality, ilvl, minLvl, iType = TSMAPI.Item:GetInfo(itemString);
		local soulbound = TSMAPI.Item:IsSoulbound(itemString);
		local usable = IsUsableItem(itemLink);
		local itemId = TSMAPI.Item:ToItemID(itemString);

		local customSalePrice = TSMAPI:GetCustomPriceValue('DBGlobalSaleAvg', itemString);

		if itemString and usable and (tContains(TDOpen_Items, itemId) or tContains(customList, itemId)) then
			if debug then
				print('Would use ' .. itemLink);
			else
				print('Using ' .. itemLink);
				TDOpenButton:SetAttribute('macrotext', format('/use %d %d', bag, slot));
				return;
			end
		end
	end
end

local TDDisFrame = CreateFrame('Frame');
TDDisFrame:RegisterEvent('PLAYER_ENTERING_WORLD')
TDDisFrame:SetScript('OnEvent', function(self, event, ...)
	TDDisButton = CreateFrame('Button', 'TDDisButton', nil, 'SecureActionButtonTemplate');
	TDDisButton:SetScript('PreClick', function()
		TDAutoDisenchant(TD_debug, TD_maxValue, TD_valueDiff, TD_priceSource, TD_maxQuality, TD_maxIlvl);
	end);
	TDDisButton:SetAttribute('type', 'macro');

	TDOpenButton = CreateFrame('Button', 'TDOpenButton', nil, 'SecureActionButtonTemplate');
	TDOpenButton:SetScript('PreClick', function()
		TDAutoOpen(TD_debug, TD_customOpenList);
	end);
	TDOpenButton:SetAttribute('type', 'macro');
end)
